import { Injectable } from '@angular/core';
import {
  HttpErrorResponse,
  HttpEvent,
  HttpHandler,
  HttpInterceptor,
  HttpRequest,
  HttpResponse
} from '@angular/common/http';
import {Observable, of} from 'rxjs';
import {catchError, finalize, tap} from 'rxjs/operators';
import {ToastrService} from 'ngx-toastr';
import {NotificationService} from '../../core/services/notification.service';
import {NgxUiLoaderService} from 'ngx-ui-loader';
import {FileService} from 'projects/pastis/src/app/core/services/file.service';
import {SedaService} from 'projects/pastis/src/app/core/services/seda.service';
import {TranslateService} from "@ngx-translate/core";

const PASTIS_SPINNER_TRANSLATE_PATH = 'PASTIS_SPINNER_INTERCEPTOR';

@Injectable({
  providedIn: 'root'
})



export class PastisSpinnerInterceptor implements HttpInterceptor {

  count = 0;
  isSedaLoaded: boolean;

  /* notificationCreateProfile = $localize`:@@notification.CreateProfile:Les données RNG ont été chargées avec succès`;
   notificationEdit = $localize`:@@notification.Edit:Les données RNG ont été chargées avec succès`;
   notificationSeda = $localize`:@@notification.Seda:Les données SEDA ont été chargées avec succès`;
   notificationUpdateProfile = $localize`:@@notification.UpdateProfile:Le fichier RNG a été généré avec succès`;*/

  notificationCreateProfile: string = this.translated('.NOTIFICATION_CREATE_PROFILE');
  notificationEdit: string = this.translated('.NOTIFICATION_EDIT');
  notificationSeda: string = this.translated('.NOTIFICATION_SEDA');
  notificationUpdateProfile: string = this.translated('.NOTIFICATION_UPDATE_PROFILE');


  constructor(public ngxService: NgxUiLoaderService,
              public notificationService: NotificationService,
              public toasterService: ToastrService,
              public translateService: TranslateService) {
  }

  intercept(req: HttpRequest<FileService | SedaService>, next: HttpHandler): Observable<HttpEvent<any>> {

    setTimeout(() => {
      if (this.count < 0) return next.handle(req);
      this.ngxService.start();
      this.count++;
    }, 10);


    return next.handle(req).pipe(
      tap(evt => {
        if (evt instanceof HttpResponse && new RegExp('createprofile').test(req.url)) {
          if (evt.body) {
            console.log('Inside loaderInterceptor on url : ', req.url);
            this.notificationService.showSuccess(this.notificationCreateProfile);
          }
        }
        if (evt instanceof HttpResponse && new RegExp('edit').test(req.url)) {
          if (evt.body) {
            console.log('Inside loaderInterceptor on url : ', req.url);
            this.notificationService.showSuccess(this.notificationEdit);
          }
        }
        if (evt instanceof HttpResponse && new RegExp('seda').test(req.url)) {
          if (evt.body && !this.isSedaLoaded) {
            console.log('Inside loaderInterceptor on url : ', req.url);
            this.notificationService.showSuccess(this.notificationSeda);
            this.isSedaLoaded = true;
          }
        }
        if (evt instanceof HttpResponse && new RegExp('updateprofile').test(req.url)) {
          if (evt.body) {
            console.log('Inside loaderInterceptor on url : ', req.url);
            this.notificationService.showSuccess(this.notificationUpdateProfile);
          }
        }
      }),
      finalize(() => {
        this.count--;
        if (this.count === 0) this.ngxService.stop();
      }),
      catchError((error: any) => {
        if (error instanceof HttpErrorResponse) {
          try {
            this.handleError(error);
          } catch (error) {
            this.handleError(error);
          }
          //log error
        }
        return of(error);
      }));

  }

  translated(nameOfFieldToTranslate: string): string {
    return this.translateService.instant(PASTIS_SPINNER_TRANSLATE_PATH + nameOfFieldToTranslate);
  }


  handleError(error: HttpErrorResponse) {
    if (new RegExp('createprofile').test(error.url)) this.notificationService.showError('Échec lors du chargement du fichier RNG');
    if (new RegExp('seda').test(error.url)) this.notificationService.showError('Échec lors du chargement des données SEDA');
    if (new RegExp('updateprofile').test(error.url)) this.notificationService.showError('Échec lors de la géneration du fichier RNG');
  }

}
